﻿using System.Linq.Expressions;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Extensions;
using SqlSugar;
using XF.Domain.Entities;
using XF.Domain.Repository;

namespace Hotline.Repository.SqlSugar;

public class BaseRepositoryWorkflow<TEntity> : BaseRepository<TEntity>, IRepositoryWorkflow<TEntity>
    where TEntity : class, IEntity<string>, IHasCreationTime, IDataPermission, IWorkflow, new()
{
    private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;

    protected BaseRepositoryWorkflow(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder) : base(uow, dataPermissionFilterBuilder)
    {
        _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
    }

    public ISugarQueryable<TEntity> Queryable(bool permissionVerify = false, bool includeDeleted = false, bool viewFilter = true, bool? handlerFilter = null)
    {
        if (includeDeleted)
            Db.QueryFilter.Clear();

        var query = Db.Queryable<TEntity>();
        if (permissionVerify)
            query = query.DataPermissionFiltering(_dataPermissionFilterBuilder);

        if (viewFilter)
            query = query.WorkflowViewFiltering(_dataPermissionFilterBuilder);

        if (handlerFilter.HasValue)
            query = query.WorkflowHandleFiltering(_dataPermissionFilterBuilder, handlerFilter.Value);
        
        return query;
    }

    public async Task<string> AddAsync(TEntity entity, CancellationToken cancellationToken = default)
    {
        entity.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager)
            .AssignToCreator(_dataPermissionFilterBuilder.DataPermissionManager);
        var excEntity = await Db.Insertable(entity).ExecuteReturnEntityAsync();
        return excEntity.Id;
    }

    /// <summary>
    /// 批量插入（应用场景：小数据量，超出1万条建议另行实现）
    /// </summary>
    /// <param name="entities"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task AddRangeAsync(List<TEntity> entities, CancellationToken cancellationToken = default)
    {
        entities.ForEach(d => d.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager)
                .AssignToCreator(_dataPermissionFilterBuilder.DataPermissionManager));
        await Db.Insertable(entities).ExecuteCommandAsync();
    }
}